import datetime
from fastapi import Depends
from fastapi.testclient import TestClient
from numpy.testing import assert_equal
from obspy import UTCDateTime
import pytest

from geomagio.api.ws import app
from geomagio.api.ws.filter import get_filter_data_query
from geomagio.api.ws.FilterApiQuery import FilterApiQuery
from geomagio.api.ws.DataApiQuery import SamplingPeriod, OutputFormat, DataType


@pytest.fixture(scope="module")
def test_client():
    @app.get("/filter/", response_model=FilterApiQuery)
    def get_query(query: FilterApiQuery = Depends(get_filter_data_query)):
        return query

    client = TestClient(app)
    yield client


def test_get_filter_data_query(test_client):
    """test.api_test.ws_test.filter_test.test_get_filter_data_query()"""
    response = test_client.get(
        "/filter/?id=ANMO&starttime=2020-09-01T00:00:01&elements=X,Y,Z,F&type=variation&input_sampling_period=60&output_sampling_period=3600&format=iaga2002"
    )
    query = FilterApiQuery(**response.json())
    assert_equal(query.id, "ANMO")
    assert_equal(query.starttime, UTCDateTime("2020-09-01T00:00:01"))
    assert_equal(query.endtime, UTCDateTime("2020-09-02T00:00:00.999"))
    assert_equal(query.elements, ["X", "Y", "Z", "F"])
    assert_equal(query.format, "iaga2002")
    assert_equal(query.data_type, "variation")
    assert_equal(query.input_sampling_period, SamplingPeriod.MINUTE)
    assert_equal(query.sampling_period, SamplingPeriod.HOUR)


def test_get_filter_data_query_no_starttime(test_client):
    """test.api_test.ws_test.data_test.test_get_filter_data_query_no_starttime()"""
    response = test_client.get("/filter/?id=BOU&elements=X,Y,Z,F")
    query = FilterApiQuery(**response.json())
    assert_equal(query.id, "BOU")

    # expect starttime to default to start of current day
    now = datetime.datetime.now(tz=datetime.timezone.utc)
    expected = UTCDateTime(year=now.year, month=now.month, day=now.day)

    assert_equal(query.starttime, expected)
    # expect endtime to default to 1 day after starttiime
    assert_equal(query.endtime, expected + (86400 - 0.001))
